==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/item/equipment/tool/ThundercallerItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.tool;

import it.unimi.dsi.fastutil.ints.IntArrayList;
import it.unimi.dsi.fastutil.ints.IntList;
import org.jetbrains.annotations.NotNull;

import vazkii.botania.api.BotaniaAPI;
import vazkii.botania.common.item.equipment.tool.manasteel.ManasteelSwordItem;
import vazkii.botania.network.EffectType;
import vazkii.botania.network.clientbound.BotaniaEffectPacket;
import vazkii.botania.xplat.XplatAbstractions;

import java.util.List;
import java.util.function.Predicate;
import net.minecraft.class_1282;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1569;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_238;

public class ThundercallerItem extends ManasteelSwordItem {
	public ThundercallerItem(class_1793 props) {
		super(BotaniaAPI.instance().getTerrasteelItemTier(), 3, -1.5F, props);
	}

	@Override
	public boolean method_7873(class_1799 stack, class_1309 entity, @NotNull class_1309 attacker) {
		double range = 8;
		IntList alreadyTargetedEntities = new IntArrayList();

		Predicate<class_1297> selector = e -> e instanceof class_1309 && e instanceof class_1569 && !(e instanceof class_1657) && !alreadyTargetedEntities.contains(e.method_5628());

		class_1309 prevTarget = entity;
		int hops = entity.field_6002.method_8546() ? 10 : 4;
		int dmg = hops + 1;
		for (int i = 0; i < hops; i++) {
			List<class_1297> entities = entity.field_6002.method_8333(prevTarget, new class_238(prevTarget.method_23317() - range, prevTarget.method_23318() - range, prevTarget.method_23321() - range, prevTarget.method_23317() + range, prevTarget.method_23318() + range, prevTarget.method_23321() + range), selector);
			if (entities.isEmpty()) {
				break;
			}

			class_1309 target = (class_1309) entities.get(entity.field_6002.method_8409().method_43048(entities.size()));
			if (attacker instanceof class_1657 player) {
				target.method_5643(class_1282.method_5532(player), dmg);
			} else {
				target.method_5643(class_1282.method_5511(attacker), dmg);
			}

			alreadyTargetedEntities.add(target.method_5628());
			prevTarget = target;
			dmg--;
		}

		if (!alreadyTargetedEntities.isEmpty()) {
			XplatAbstractions.INSTANCE.sendToTracking(attacker,
					new BotaniaEffectPacket(EffectType.THUNDERCALLER_EFFECT,
							attacker.method_23317(), attacker.method_23318() + attacker.method_17682() / 2.0, attacker.method_23321(),
							alreadyTargetedEntities.toArray(new int[0])));
		}

		return super.method_7873(stack, entity, attacker);
	}

}

==> Botania-1.20.1-451-FABRIC-20251018.024823-16-sources.jar: vazkii/botania/common/item/equipment/tool/ThundercallerItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.tool;

import it.unimi.dsi.fastutil.ints.IntArrayList;
import it.unimi.dsi.fastutil.ints.IntList;
import org.jetbrains.annotations.NotNull;

import vazkii.botania.api.BotaniaAPI;
import vazkii.botania.common.item.equipment.tool.manasteel.ManasteelSwordItem;
import vazkii.botania.network.EffectType;
import vazkii.botania.network.clientbound.BotaniaEffectPacket;
import vazkii.botania.xplat.XplatAbstractions;

import java.util.List;
import java.util.function.Predicate;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1569;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_238;

public class ThundercallerItem extends ManasteelSwordItem {
	public ThundercallerItem(class_1793 props) {
		super(BotaniaAPI.instance().getTerrasteelItemTier(), 3, -1.5F, props);
	}

	@Override
	public boolean method_7873(class_1799 stack, class_1309 entity, @NotNull class_1309 attacker) {
		double range = 8;
		IntList alreadyTargetedEntities = new IntArrayList();

		Predicate<class_1297> selector = e -> e instanceof class_1309 && e instanceof class_1569 && !(e instanceof class_1657) && !alreadyTargetedEntities.contains(e.method_5628());

		class_1309 prevTarget = entity;
		int hops = entity.method_37908().method_8546() ? 10 : 4;
		int dmg = hops + 1;
		for (int i = 0; i < hops; i++) {
			List<class_1297> entities = entity.method_37908().method_8333(prevTarget, new class_238(prevTarget.method_23317() - range, prevTarget.method_23318() - range, prevTarget.method_23321() - range, prevTarget.method_23317() + range, prevTarget.method_23318() + range, prevTarget.method_23321() + range), selector);
			if (entities.isEmpty()) {
				break;
			}

			class_1309 target = (class_1309) entities.get(entity.method_37908().method_8409().method_43048(entities.size()));
			if (attacker instanceof class_1657 player) {
				target.method_5643(player.method_48923().method_48802(player), dmg);
			} else {
				target.method_5643(attacker.method_48923().method_48812(attacker), dmg);
			}

			alreadyTargetedEntities.add(target.method_5628());
			prevTarget = target;
			dmg--;
		}

		if (!alreadyTargetedEntities.isEmpty()) {
			XplatAbstractions.INSTANCE.sendToTracking(attacker,
					new BotaniaEffectPacket(EffectType.THUNDERCALLER_EFFECT,
							attacker.method_23317(), attacker.method_23318() + attacker.method_17682() / 2.0, attacker.method_23321(),
							alreadyTargetedEntities.toArray(new int[0])));
		}

		return super.method_7873(stack, entity, attacker);
	}

}
